@@ -10,23 +10,12 @@ from cola import core
 from cola import gitcmds
 from cola import gitcfg
 from cola import textwrap
+from cola import qtutils
 from cola.cmds import Interaction
 from cola.gitcmds import commit_message_path
 from cola.i18n import N_
-from cola.models.dag import DAG
-from cola.models.dag import RepoReader
-from cola.models.prefs import tabwidth
-from cola.models.prefs import textwidth
-from cola.models.prefs import linebreak
-from cola.qtutils import add_action
-from cola.qtutils import confirm
-from cola.qtutils import connect_action_bool
-from cola.qtutils import connect_button
-from cola.qtutils import create_toolbutton
-from cola.qtutils import diff_font
-from cola.qtutils import hide_button_menu_indicator
-from cola.qtutils import options_icon
-from cola.qtutils import save_icon
+from cola.models import dag
+from cola.models import prefs
 from cola.widgets import defs
 from cola.widgets.selectcommits import select_commits
 from cola.widgets.spellcheck import SpellCheckTextEdit
@@ -39,7 +28,6 @@ class CommitMessageEditor(QtGui.QWidget):
         QtGui.QWidget.__init__(self, parent)
 
         self.model = model
-        self.notifying = False
         self.spellcheck_initialized = False
 
         self._linebreak = None
@@ -47,39 +35,44 @@ class CommitMessageEditor(QtGui.QWidget):
         self._tabwidth = None
 
         # Actions
-        self.signoff_action = add_action(self, cmds.SignOff.name(),
-                                         cmds.run(cmds.SignOff),
-                                         cmds.SignOff.SHORTCUT)
+        self.signoff_action = qtutils.add_action(self, cmds.SignOff.name(),
+                                                 cmds.run(cmds.SignOff),
+                                                 cmds.SignOff.SHORTCUT)
         self.signoff_action.setToolTip(N_('Sign off on this commit'))
 
-        self.commit_action = add_action(self,
-                                        N_('Commit@@verb'),
-                                        self.commit,
-                                        cmds.Commit.SHORTCUT)
+        self.commit_action = qtutils.add_action(self,
+                                                N_('Commit@@verb'),
+                                                self.commit,
+                                                cmds.Commit.SHORTCUT)
         self.commit_action.setToolTip(N_('Commit staged changes'))
+        self.clear_action = qtutils.add_action(self, N_('Clear...'), self.clear)
 
         # Widgets
         self.summary = CommitSummaryLineEdit()
         self.summary.setMinimumHeight(defs.tool_button_height)
+        self.summary.extra_actions.append(self.clear_action)
+        self.summary.extra_actions.append(None)
         self.summary.extra_actions.append(self.signoff_action)
         self.summary.extra_actions.append(self.commit_action)
 
         self.description = CommitMessageTextEdit()
+        self.description.extra_actions.append(self.clear_action)
+        self.description.extra_actions.append(None)
         self.description.extra_actions.append(self.signoff_action)
         self.description.extra_actions.append(self.commit_action)
 
         commit_button_tooltip = N_('Commit staged changes\n'
                                    'Shortcut: Ctrl+Enter')
-        self.commit_button = create_toolbutton(text=N_('Commit@@verb'),
-                                               tooltip=commit_button_tooltip,
-                                               icon=save_icon())
+        self.commit_button = qtutils.create_toolbutton(
+                text=N_('Commit@@verb'), tooltip=commit_button_tooltip,
+                icon=qtutils.save_icon())
 
         self.actions_menu = QtGui.QMenu()
-        self.actions_button = create_toolbutton(icon=options_icon(),
-                                                tooltip=N_('Actions...'))
+        self.actions_button = qtutils.create_toolbutton(
+                icon=qtutils.options_icon(), tooltip=N_('Actions...'))
         self.actions_button.setMenu(self.actions_menu)
         self.actions_button.setPopupMode(QtGui.QToolButton.InstantPopup)
-        hide_button_menu_indicator(self.actions_button)
+        qtutils.hide_button_menu_indicator(self.actions_button)
 
         self.actions_menu.addAction(self.signoff_action)
         self.actions_menu.addAction(self.commit_action)
@@ -92,6 +85,19 @@ class CommitMessageEditor(QtGui.QWidget):
         self.amend_action.setShortcut(cmds.AmendMode.SHORTCUT)
         self.amend_action.setShortcutContext(Qt.ApplicationShortcut)
 
+        # Bypass hooks
+        self.bypass_commit_hooks_action = self.actions_menu.addAction(
+                N_('Bypass Commit Hooks'))
+        self.bypass_commit_hooks_action.setCheckable(True)
+        self.bypass_commit_hooks_action.setChecked(False)
+
+        # Sign commits
+        cfg = gitcfg.current()
+        self.sign_action = self.actions_menu.addAction(
+                N_('Create Signed Commit'))
+        self.sign_action.setCheckable(True)
+        self.sign_action.setChecked(cfg.get('cola.signcommits', False))
+
         # Spell checker
         self.check_spelling_action = self.actions_menu.addAction(
                 N_('Check Spelling'))
@@ -102,7 +108,7 @@ class CommitMessageEditor(QtGui.QWidget):
         self.autowrap_action = self.actions_menu.addAction(
                 N_('Auto-Wrap Lines'))
         self.autowrap_action.setCheckable(True)
-        self.autowrap_action.setChecked(linebreak())
+        self.autowrap_action.setChecked(prefs.linebreak())
 
         # Commit message
         self.actions_menu.addSeparator()
@@ -116,37 +122,38 @@ class CommitMessageEditor(QtGui.QWidget):
         self.connect(self.fixup_commit_menu, SIGNAL('aboutToShow()'),
                      self.build_fixup_menu)
 
-        self.toplayout = QtGui.QHBoxLayout()
-        self.toplayout.setMargin(defs.no_margin)
+        self.toplayout = qtutils.hbox(defs.no_margin, defs.spacing,
+                                      self.actions_button, self.summary,
+                                      self.commit_button)
         self.toplayout.setContentsMargins(defs.margin, defs.no_margin,
                                           defs.no_margin, defs.no_margin)
-        self.toplayout.setSpacing(defs.spacing)
-        self.toplayout.addWidget(self.actions_button)
-        self.toplayout.addWidget(self.summary)
-        self.toplayout.addWidget(self.commit_button)
 
-        self.mainlayout = QtGui.QVBoxLayout()
-        self.mainlayout.setMargin(defs.no_margin)
-        self.mainlayout.setSpacing(defs.spacing)
-        self.mainlayout.addLayout(self.toplayout)
-        self.mainlayout.addWidget(self.description)
+        self.mainlayout = qtutils.vbox(defs.no_margin, defs.spacing,
+                                       self.toplayout, self.description)
         self.setLayout(self.mainlayout)
 
-        connect_button(self.commit_button, self.commit)
+        qtutils.connect_button(self.commit_button, self.commit)
 
         # Broadcast the amend mode
-        connect_action_bool(self.amend_action, cmds.run(cmds.AmendMode))
-        connect_action_bool(self.check_spelling_action,
-                            self.toggle_check_spelling)
+        qtutils.connect_action_bool(self.amend_action, cmds.run(cmds.AmendMode))
+        qtutils.connect_action_bool(self.check_spelling_action,
+                                    self.toggle_check_spelling)
 
         # Handle the one-off autowrapping
-        connect_action_bool(self.autowrap_action, self.set_linebreak)
+        qtutils.connect_action_bool(self.autowrap_action, self.set_linebreak)
 
-        add_action(self.summary, N_('Move Down'), self.focus_description,
-                Qt.Key_Down, Qt.Key_Return, Qt.Key_Enter)
+        qtutils.add_action(self.summary, N_('Move Down'),
+                           self.focus_description,
+                           Qt.Key_Return, Qt.Key_Enter)
+
+        qtutils.add_action(self.summary, N_('Move Down'),
+                           self.summary_cursor_down,
+                           Qt.Key_Down)
 
         self.model.add_observer(self.model.message_commit_message_changed,
-                                self.set_commit_message)
+                                self._set_commit_message)
+        self.connect(self, SIGNAL('set_commit_message(PyQt_PyObject)'),
+                     self.set_commit_message, Qt.QueuedConnection)
 
         self.connect(self.summary, SIGNAL('cursorPosition(int,int)'),
                      self.emit_position)
@@ -165,7 +172,7 @@ class CommitMessageEditor(QtGui.QWidget):
         self.connect(self.description, SIGNAL('leave()'),
                      self.focus_summary)
 
-        self.setFont(diff_font())
+        self.setFont(qtutils.diff_font())
 
         self.summary.enable_hint(True)
         self.description.enable_hint(True)
@@ -175,9 +182,9 @@ class CommitMessageEditor(QtGui.QWidget):
 
         self.setFocusProxy(self.summary)
 
-        self.set_tabwidth(tabwidth())
-        self.set_textwidth(textwidth())
-        self.set_linebreak(linebreak())
+        self.set_tabwidth(prefs.tabwidth())
+        self.set_textwidth(prefs.textwidth())
+        self.set_linebreak(prefs.linebreak())
 
         # Loading message
         commit_msg = ''
@@ -202,6 +209,21 @@ class CommitMessageEditor(QtGui.QWidget):
     def focus_description(self):
         self.description.setFocus()
 
+    def summary_cursor_down(self):
+        """Handle the down key in the summary field
+
+        If the cursor is at the end of the line then focus the description.
+        Otherwise, move the cursor to the end of the line so that a
+        subsequence "down" press moves to the end of the line.
+
+        """
+        cur_position = self.summary.cursorPosition()
+        end_position = len(self.summary.value())
+        if cur_position == end_position:
+            self.focus_description()
+        else:
+            self.summary.setCursorPosition(end_position)
+
     def commit_message(self, raw=True):
         """Return the commit message as a unicode string"""
         summary = self.summary.value()
@@ -250,13 +272,22 @@ class CommitMessageEditor(QtGui.QWidget):
 
     def commit_message_changed(self, value=None):
         """Update the model when values change"""
-        self.notifying = True
         message = self.commit_message()
-        self.model.set_commitmsg(message)
+        self.model.set_commitmsg(message, notify=False)
         self.refresh_palettes()
-        self.notifying = False
         self.update_actions()
 
+    def clear(self):
+        if not qtutils.confirm(
+                N_('Clear commit message?'),
+                N_('The commit message will be cleared.'),
+                N_('This cannot be undone.  Clear commit message?'),
+                N_('Clear commit message'),
+                default=True,
+                icon=qtutils.discard_icon()):
+            return
+        self.model.set_commitmsg('')
+
     def update_actions(self):
         commit_enabled = bool(self.summary.value())
         self.commit_button.setEnabled(commit_enabled)
@@ -267,13 +298,11 @@ class CommitMessageEditor(QtGui.QWidget):
         self.summary.refresh_palette()
         self.description.refresh_palette()
 
+    def _set_commit_message(self, message):
+        self.emit(SIGNAL('set_commit_message(PyQt_PyObject)'), message)
+
     def set_commit_message(self, message):
         """Set the commit message to match the observed model"""
-        if self.notifying:
-            # Calling self.model.set_commitmsg(message) causes us to
-            # loop around so break the loop
-            return
-
         # Parse the "summary" and "description" fields
         umsg = ustr(message)
         lines = umsg.splitlines()
@@ -394,12 +423,12 @@ class CommitMessageEditor(QtGui.QWidget):
             if self.model.modified:
                 informative_text = N_('Would you like to stage and '
                                       'commit all modified files?')
-                if not confirm(N_('Stage and commit?'),
-                               error_msg,
-                               informative_text,
-                               N_('Stage and Commit'),
-                               default=True,
-                               icon=save_icon()):
+                if not qtutils.confirm(
+                        N_('Stage and commit?'),
+                        error_msg, informative_text,
+                        N_('Stage and Commit'),
+                        default=True,
+                        icon=qtutils.save_icon()):
                     return
             else:
                 Interaction.information(N_('Nothing to commit'), error_msg)
@@ -409,15 +438,19 @@ class CommitMessageEditor(QtGui.QWidget):
         # Warn that amending published commits is generally bad
         amend = self.amend_action.isChecked()
         if (amend and self.model.is_commit_published() and
-            not confirm(N_('Rewrite Published Commit?'),
+            not qtutils.confirm(
+                        N_('Rewrite Published Commit?'),
                         N_('This commit has already been published.\n'
                            'This operation will rewrite published history.\n'
                            'You probably don\'t want to do this.'),
                         N_('Amend the published commit?'),
                         N_('Amend Commit'),
-                        default=False, icon=save_icon())):
+                        default=False, icon=qtutils.save_icon())):
             return
-        status, out, err = cmds.do(cmds.Commit, amend, msg)
+        no_verify = self.bypass_commit_hooks_action.isChecked()
+        sign = self.sign_action.isChecked()
+        status, out, err = cmds.do(cmds.Commit, amend, msg, sign,
+                                   no_verify=no_verify)
         if status != 0:
             Interaction.critical(N_('Commit failed'),
                                  N_('"git commit" returned exit code %s') %
@@ -436,8 +469,8 @@ class CommitMessageEditor(QtGui.QWidget):
                                 self.choose_commit_message)
 
     def build_commits_menu(self, cmd, menu, chooser, prefix=''):
-        dag = DAG('HEAD', 6)
-        commits = RepoReader(dag)
+        ctx = dag.DAG('HEAD', 6)
+        commits = dag.RepoReader(ctx)
 
         menu_commits = []
         for idx, c in enumerate(commits):
@@ -475,7 +508,7 @@ class CommitMessageEditor(QtGui.QWidget):
         if enabled and not self.spellcheck_initialized:
             # Add our name to the dictionary
             self.spellcheck_initialized = True
-            cfg = gitcfg.instance()
+            cfg = gitcfg.current()
             user_name = cfg.get('user.name')
             if user_name:
                 for part in user_name.split():
@@ -503,12 +536,19 @@ class CommitSummaryLineEdit(HintedLineEdit):
         HintedLineEdit.__init__(self, hint, parent)
         self.extra_actions = []
 
+        regex = QtCore.QRegExp(r'^[^#].*')
+        self._validator = QtGui.QRegExpValidator(regex, self)
+        self.setValidator(self._validator)
+
     def contextMenuEvent(self, event):
         menu = self.createStandardContextMenu()
         if self.extra_actions:
             menu.addSeparator()
         for action in self.extra_actions:
-            menu.addAction(action)
+            if action is None:
+                menu.addSeparator()
+            else:
+                menu.addAction(action)
         menu.exec_(self.mapToGlobal(event.pos()))
 
 
@@ -519,7 +559,7 @@ class CommitMessageTextEdit(SpellCheckTextEdit):
         SpellCheckTextEdit.__init__(self, hint, parent)
         self.extra_actions = []
 
-        self.action_emit_leave = add_action(self,
+        self.action_emit_leave = qtutils.add_action(self,
                 'Shift Tab', self.emit_leave, 'Shift+tab')
 
     def contextMenuEvent(self, event):
@@ -527,7 +567,10 @@ class CommitMessageTextEdit(SpellCheckTextEdit):
         if self.extra_actions:
             menu.addSeparator()
         for action in self.extra_actions:
-            menu.addAction(action)
+            if action is None:
+                menu.addSeparator()
+            else:
+                menu.addAction(action)
         menu.exec_(self.mapToGlobal(event.pos()))
 
     def keyPressEvent(self, event):
