@@ -5,10 +5,12 @@ from PyQt4.QtCore import SIGNAL
 
 from cola.i18n import N_
 from cola.git import git
+from cola import qtutils
 from cola.widgets.standard import TreeWidget
 from cola.widgets.diff import COMMITS_SELECTED
 from cola.widgets.diff import FILES_SELECTED
 
+HISTORIES_SELECTED = 'HISTORIES_SELECTED'
 
 class FileWidget(TreeWidget):
 
@@ -31,10 +33,10 @@ class FileWidget(TreeWidget):
             return
         commit = commits[0]
         sha1 = commit.sha1
-        status, out, err = git.show(sha1,
-                                    numstat=True, oneline=True, no_renames=True)
+        status, out, err = git.show(sha1, z=True, numstat=True,
+                                    oneline=True, no_renames=True)
         if status == 0:
-            paths = [f for f in out.splitlines() if f]
+            paths = [f for f in out.rstrip('\0').split('\0') if f]
             if paths:
                 paths = paths[1:]
         else:
@@ -52,9 +54,9 @@ class FileWidget(TreeWidget):
         self.insertTopLevelItems(0, files)
 
     def adjust_columns(self):
-        width = self.width()-20
-        zero = width*2//3
-        onetwo = width//6
+        width = self.width() - 20
+        zero = width*2 // 3
+        onetwo = width // 6
         self.setColumnWidth(0, zero)
         self.setColumnWidth(1, onetwo)
         self.setColumnWidth(2, onetwo)
@@ -65,6 +67,16 @@ class FileWidget(TreeWidget):
     def resizeEvent(self, e):
         self.adjust_columns()
 
+    def contextMenuEvent(self, event):
+        menu = QtGui.QMenu(self)
+        menu.addAction(qtutils.add_action(self, N_('Show history'),
+                               self.show_file_history))
+        menu.exec_(self.mapToGlobal(event.pos()))
+
+    def show_file_history(self):
+        items = self.selected_items()
+        self.notifier.notify_observers(HISTORIES_SELECTED,
+                                       [i.file_name for i in items])
 
 class FileTreeWidgetItem(QtGui.QTreeWidgetItem):
 
