@@ -57,7 +57,8 @@ class GrepThread(QtCore.QThread):
             args = [query]
         status, out, err = git.grep(self.regexp_mode, n=True, *args)
         if query == self.query:
-            self.emit(SIGNAL('result'), status, out, err)
+            self.emit(SIGNAL('result(PyQt_PyObject,PyQt_PyObject,PyQt_PyObject)'),
+                      status, out, err)
         else:
             self.run()
 
@@ -116,37 +117,24 @@ class Grep(Dialog):
 
         self.close_button = QtGui.QPushButton(N_('Close'))
 
-        self.input_layout = QtGui.QHBoxLayout()
-        self.input_layout.setMargin(defs.no_margin)
-        self.input_layout.setSpacing(defs.button_spacing)
+        self.input_layout = qtutils.hbox(defs.no_margin, defs.button_spacing,
+                                         self.input_label, self.input_txt,
+                                         self.regexp_combo)
 
-        self.bottom_layout = QtGui.QHBoxLayout()
-        self.bottom_layout.setMargin(defs.no_margin)
-        self.bottom_layout.setSpacing(defs.button_spacing)
+        self.bottom_layout = qtutils.hbox(defs.no_margin, defs.button_spacing,
+                                          self.edit_button, self.refresh_button,
+                                          self.shell_checkbox, qtutils.STRETCH,
+                                          self.close_button)
 
-        self.mainlayout = QtGui.QVBoxLayout()
-        self.mainlayout.setMargin(defs.margin)
-        self.mainlayout.setSpacing(defs.spacing)
-
-        self.input_layout.addWidget(self.input_label)
-        self.input_layout.addWidget(self.input_txt)
-        self.input_layout.addWidget(self.regexp_combo)
-
-        self.bottom_layout.addWidget(self.edit_button)
-        self.bottom_layout.addWidget(self.refresh_button)
-        self.bottom_layout.addWidget(self.shell_checkbox)
-        self.bottom_layout.addStretch()
-        self.bottom_layout.addWidget(self.close_button)
-
-        self.mainlayout.addLayout(self.input_layout)
-        self.mainlayout.addWidget(self.result_txt)
-        self.mainlayout.addLayout(self.bottom_layout)
+        self.mainlayout = qtutils.vbox(defs.margin, defs.no_spacing,
+                                       self.input_layout, self.result_txt,
+                                       self.bottom_layout)
         self.setLayout(self.mainlayout)
 
-        self.grep_thread = GrepThread(self)
-
-        self.connect(self.grep_thread, SIGNAL('result'),
-                     self.process_result)
+        self.worker_thread = GrepThread(self)
+        self.connect(self.worker_thread,
+                     SIGNAL('result(PyQt_PyObject,PyQt_PyObject,PyQt_PyObject)'),
+                     self.process_result, Qt.QueuedConnection)
 
         self.connect(self.input_txt, SIGNAL('textChanged(QString)'),
                      lambda s: self.search())
@@ -157,12 +145,10 @@ class Grep(Dialog):
         self.connect(self.result_txt, SIGNAL('leave()'),
                      lambda: self.input_txt.setFocus())
 
-        qtutils.add_action(self.input_txt, 'FocusResults',
-                           lambda: self.result_txt.setFocus(),
+        qtutils.add_action(self.input_txt, 'Focus Results', self.focus_results,
                            Qt.Key_Down, Qt.Key_Enter, Qt.Key_Return)
-        qtutils.add_action(self, 'FocusSearch',
-                           lambda: self.input_txt.setFocus(),
-                           'Ctrl+L')
+        qtutils.add_action(self, 'Focus Input', self.focus_input, 'Ctrl+L')
+
         qtutils.connect_button(self.edit_button, self.edit)
         qtutils.connect_button(self.refresh_button, self.search)
         qtutils.connect_toggle(self.shell_checkbox, lambda x: self.search())
@@ -170,7 +156,14 @@ class Grep(Dialog):
         qtutils.add_close_action(self)
 
         if not self.restore_state():
-            self.resize(666, 420)
+            width, height = qtutils.default_size(parent, 666, 420)
+            self.resize(width, height)
+
+    def focus_input(self):
+        self.input_txt.setFocus()
+
+    def focus_results(self):
+        self.result_txt.setFocus()
 
     def done(self, exit_code):
         self.save_state()
@@ -188,14 +181,13 @@ class Grep(Dialog):
         if len(query) < 2:
             self.result_txt.set_value('')
             return
-        self.grep_thread.query = query
-        self.grep_thread.shell = self.shell_checkbox.isChecked()
-        self.grep_thread.regexp_mode = self.regexp_mode()
-        self.grep_thread.start()
+        self.worker_thread.query = query
+        self.worker_thread.shell = self.shell_checkbox.isChecked()
+        self.worker_thread.regexp_mode = self.regexp_mode()
+        self.worker_thread.start()
 
     def search_for(self, txt):
         self.input_txt.set_value(txt)
-        self.search()
 
     def text_scroll(self):
         scrollbar = self.result_txt.verticalScrollBar()
@@ -315,15 +307,6 @@ class GrepTextView(HintedTextView):
         if cursor.movePosition(direction):
             self.set_text_cursor(cursor)
 
-    def paintEvent(self, event):
-        HintedTextView.paintEvent(self, event)
-        if self.hasFocus():
-            # Qt doesn't redraw the cursor when using movePosition().
-            # So.. draw our own cursor.
-            rect = self.cursorRect()
-            painter = QtGui.QPainter(self.viewport())
-            painter.fillRect(rect, Qt.SolidPattern)
-
     def keyPressEvent(self, event):
         if event.key() == Qt.Key_Up:
             cursor = self.textCursor()
@@ -334,4 +317,7 @@ class GrepTextView(HintedTextView):
                 # Otherwise, emit a signal so that the parent can
                 # change focus.
                 self.emit(SIGNAL('leave()'))
+            elif self.value()[:position].count('\n') == 0:
+                cursor.movePosition(QtGui.QTextCursor.StartOfLine)
+                self.setTextCursor(cursor)
         return HintedTextView.keyPressEvent(self, event)
