@@ -5,6 +5,7 @@ from PyQt4.QtCore import Qt
 from PyQt4.QtCore import SIGNAL
 
 from cola import cmds
+from cola import gitcfg
 from cola import gitcmds
 from cola import qtutils
 from cola.i18n import N_
@@ -17,7 +18,8 @@ from cola.compat import ustr
 def local_merge():
     """Provides a dialog for merging branches"""
     model = main.model()
-    view = MergeView(model, qtutils.active_window())
+    cfg = gitcfg.current()
+    view = MergeView(cfg, model, qtutils.active_window())
     view.show()
     view.raise_()
     return view
@@ -33,15 +35,16 @@ def abort_merge():
     info_txt = N_('Aborting the current merge?')
     ok_txt = N_('Abort Merge')
     if qtutils.confirm(title, txt, info_txt, ok_txt,
-                       default=False, icon=qtutils.icon('undo.svg')):
+                       default=False, icon=qtutils.theme_icon('edit-undo.svg')):
         gitcmds.abort_merge()
 
 
 class MergeView(QtGui.QDialog):
     """Provides a dialog for merging branches."""
 
-    def __init__(self, model, parent=None):
+    def __init__(self, cfg, model, parent=None):
         QtGui.QDialog.__init__(self, parent)
+        self.cfg = cfg
         self.model = model
         if parent is not None:
             self.setWindowModality(Qt.WindowModal)
@@ -50,76 +53,80 @@ class MergeView(QtGui.QDialog):
         # Widgets
         self.title_label = QtGui.QLabel()
         self.revision_label = QtGui.QLabel()
-        self.revision_label.setText(N_('Revision To Merge'))
+        self.revision_label.setText(N_('Revision to Merge'))
 
         self.revision = completion.GitRefLineEdit()
+        self.revision.setFocus()
+        self.revision.setToolTip(N_('Revision to Merge'))
 
         self.radio_local = QtGui.QRadioButton()
         self.radio_local.setText(N_('Local Branch'))
         self.radio_local.setChecked(True)
+
         self.radio_remote = QtGui.QRadioButton()
         self.radio_remote.setText(N_('Tracking Branch'))
+
         self.radio_tag = QtGui.QRadioButton()
         self.radio_tag.setText(N_('Tag'))
 
         self.revisions = QtGui.QListWidget()
         self.revisions.setAlternatingRowColors(True)
 
-        self.button_viz = QtGui.QPushButton(self)
+        self.button_viz = QtGui.QPushButton()
         self.button_viz.setText(N_('Visualize'))
 
-        self.checkbox_squash = QtGui.QCheckBox(self)
+        tooltip = N_('Squash the merged commit(s) into a single commit')
+        self.checkbox_squash = QtGui.QCheckBox()
         self.checkbox_squash.setText(N_('Squash'))
+        self.checkbox_squash.setToolTip(tooltip)
 
-        self.checkbox_noff = QtGui.QCheckBox(self)
+        tooltip = N_('Always create a merge commit when enabled, '
+                     'even when the merge is a fast-forward update')
+        self.checkbox_noff = QtGui.QCheckBox()
         self.checkbox_noff.setText(N_('No fast forward'))
+        self.checkbox_noff.setToolTip(tooltip)
         self.checkbox_noff.setChecked(False)
         self.checkbox_noff_state = False
 
-        self.checkbox_commit = QtGui.QCheckBox(self)
+        tooltip = N_('Commit the merge if there are no conflicts.  '
+                     'Uncheck to leave the merge uncommitted')
+        self.checkbox_commit = QtGui.QCheckBox()
         self.checkbox_commit.setText(N_('Commit'))
+        self.checkbox_commit.setToolTip(tooltip)
         self.checkbox_commit.setChecked(True)
         self.checkbox_commit_state = True
 
-        self.button_cancel = QtGui.QPushButton(self)
+        self.checkbox_sign = QtGui.QCheckBox()
+        self.checkbox_sign.setText(N_('Create Signed Commit'))
+        self.checkbox_sign.setChecked(cfg.get('cola.signcommits', False))
+        self.checkbox_sign.setToolTip(N_('GPG-sign the merge commit'))
+
+        self.button_cancel = QtGui.QPushButton()
         self.button_cancel.setText(N_('Cancel'))
 
-        self.button_merge = QtGui.QPushButton(self)
+        self.button_merge = QtGui.QPushButton()
         self.button_merge.setText(N_('Merge'))
 
         # Layouts
-        self.revlayt = QtGui.QHBoxLayout()
-        self.revlayt.addWidget(self.revision_label)
-        self.revlayt.addWidget(self.revision)
-        self.revlayt.addStretch()
-        self.revlayt.addWidget(self.title_label)
+        self.revlayt = qtutils.hbox(defs.no_margin, defs.spacing,
+                                    self.revision_label, self.revision,
+                                    qtutils.STRETCH, self.title_label)
 
-        self.radiolayt = QtGui.QHBoxLayout()
-        self.radiolayt.addWidget(self.radio_local)
-        self.radiolayt.addWidget(self.radio_remote)
-        self.radiolayt.addWidget(self.radio_tag)
+        self.radiolayt = qtutils.hbox(defs.no_margin, defs.spacing,
+                                      self.radio_local, self.radio_remote,
+                                      self.radio_tag)
 
-        self.buttonlayt = QtGui.QHBoxLayout()
-        self.buttonlayt.setSpacing(defs.button_spacing)
-        self.buttonlayt.addWidget(self.button_viz)
-        self.buttonlayt.addStretch()
-        self.buttonlayt.addWidget(self.checkbox_squash)
-        self.buttonlayt.addWidget(self.checkbox_noff)
-        self.buttonlayt.addWidget(self.checkbox_commit)
-        self.buttonlayt.addWidget(self.button_cancel)
-        self.buttonlayt.addWidget(self.button_merge)
+        self.buttonlayt = qtutils.hbox(defs.no_margin, defs.button_spacing,
+                                       self.button_viz, qtutils.STRETCH,
+                                       self.checkbox_squash, self.checkbox_noff,
+                                       self.checkbox_commit, self.checkbox_sign,
+                                       self.button_cancel, self.button_merge)
 
-        self.mainlayt = QtGui.QVBoxLayout()
-        self.mainlayt.setMargin(defs.margin)
-        self.mainlayt.setSpacing(defs.spacing)
-        self.mainlayt.addLayout(self.radiolayt)
-        self.mainlayt.addWidget(self.revisions)
-        self.mainlayt.addLayout(self.revlayt)
-        self.mainlayt.addLayout(self.buttonlayt)
+        self.mainlayt = qtutils.vbox(defs.margin, defs.spacing,
+                                     self.radiolayt, self.revisions,
+                                     self.revlayt, self.buttonlayt)
         self.setLayout(self.mainlayt)
 
-        self.revision.setFocus()
-
         # Signal/slot connections
         self.connect(self.revision, SIGNAL('textChanged(QString)'),
                      self.update_title)
@@ -159,12 +166,10 @@ class MergeView(QtGui.QDialog):
     def toggle_squash(self):
         """Toggles the commit checkbox based on the squash checkbox."""
         if self.checkbox_squash.isChecked():
-            self.checkbox_commit_state =\
-                self.checkbox_commit.checkState()
+            self.checkbox_commit_state = self.checkbox_commit.checkState()
             self.checkbox_commit.setCheckState(Qt.Unchecked)
             self.checkbox_commit.setDisabled(True)
-            self.checkbox_noff_state =\
-                self.checkbox_noff.checkState()
+            self.checkbox_noff_state = self.checkbox_noff.checkState()
             self.checkbox_noff.setCheckState(Qt.Unchecked)
             self.checkbox_noff.setDisabled(True)
         else:
@@ -184,9 +189,8 @@ class MergeView(QtGui.QDialog):
         """Update the revision field when a list item is selected"""
         revlist = self.current_revisions()
         widget = self.revisions
-        row, selected = qtutils.selected_row(widget)
-        if selected and row < len(revlist):
-            revision = revlist[row]
+        revision = qtutils.selected_item(widget, revlist)
+        if revision is not None:
             self.revision.setText(revision)
 
     def current_revisions(self):
@@ -219,5 +223,6 @@ class MergeView(QtGui.QDialog):
         noff = self.checkbox_noff.isChecked()
         do_commit = self.checkbox_commit.isChecked()
         squash = self.checkbox_squash.isChecked()
-        cmds.do(cmds.Merge, revision, not(do_commit), squash, noff)
+        sign = self.checkbox_sign.isChecked()
+        cmds.do(cmds.Merge, revision, not(do_commit), squash, noff, sign)
         self.accept()
