@@ -30,7 +30,7 @@ class UpdateFileListThread(QtCore.QThread):
     def run(self):
         ref = 'HEAD~%d' % self.count
         filenames = gitcmds.diff_index_filenames(ref)
-        self.emit(SIGNAL('filenames'), filenames)
+        self.emit(SIGNAL('filenames(PyQt_PyObject)'), filenames)
 
 
 class RecentFileDialog(standard.Dialog):
@@ -77,43 +77,34 @@ class RecentFileDialog(standard.Dialog):
         self.close_button = QtGui.QPushButton()
         self.close_button.setText(N_('Close'))
 
-        toplayout = QtGui.QHBoxLayout()
-        toplayout.setMargin(defs.no_margin)
-        toplayout.setSpacing(defs.spacing)
-        toplayout.addWidget(self.count_label)
-        toplayout.addWidget(self.count)
-        toplayout.addStretch()
-        toplayout.addWidget(self.refresh_button)
+        self.top_layout = qtutils.hbox(defs.no_margin, defs.spacing,
+                                       self.count_label, self.count,
+                                       qtutils.STRETCH, self.refresh_button)
 
-        btnlayout = QtGui.QHBoxLayout()
-        btnlayout.setMargin(defs.no_margin)
-        btnlayout.setSpacing(defs.spacing)
-        btnlayout.addWidget(self.expand_button)
-        btnlayout.addWidget(self.collapse_button)
-        btnlayout.addStretch()
-        btnlayout.addWidget(self.edit_button)
-        btnlayout.addWidget(self.close_button)
+        self.button_layout = qtutils.hbox(defs.no_margin, defs.spacing,
+                                          self.expand_button,
+                                          self.collapse_button,
+                                          qtutils.STRETCH,
+                                          self.edit_button, self.close_button)
 
-        layout = QtGui.QVBoxLayout()
-        layout.setMargin(defs.margin)
-        layout.setSpacing(defs.spacing)
-        layout.addLayout(toplayout)
-        layout.addWidget(self.tree)
-        layout.addLayout(btnlayout)
-        self.setLayout(layout)
+        self.main_layout = qtutils.vbox(defs.margin, defs.spacing,
+                                        self.top_layout, self.tree,
+                                        self.button_layout)
+        self.setLayout(self.main_layout)
 
         self.connect(self.tree, SIGNAL('selectionChanged()'),
                      self.selection_changed)
 
-        self.connect(self.tree, SIGNAL('path_chosen'), self.edit_file)
+        self.connect(self.tree, SIGNAL('path_chosen(PyQt_PyObject)'),
+                     self.edit_file)
 
         self.connect(self.count, SIGNAL('valueChanged(int)'),
                      self.count_changed)
 
         self.connect(self.count, SIGNAL('editingFinished()'), self.refresh)
 
-        self.connect(self.update_thread, SIGNAL('filenames'),
-                     self.set_filenames)
+        self.connect(self.update_thread, SIGNAL('filenames(PyQt_PyObject)'),
+                     self.set_filenames, Qt.QueuedConnection)
 
         qtutils.connect_button(self.refresh_button, self.refresh)
         qtutils.connect_button(self.expand_button, self.tree.expandAll)
