@@ -2,7 +2,7 @@
 from __future__ import division, absolute_import, unicode_literals
 
 from PyQt4 import QtGui
-from PyQt4 import QtCore
+from PyQt4.QtCore import Qt
 from PyQt4.QtCore import SIGNAL
 
 from cola import gitcmds
@@ -26,11 +26,9 @@ class Model(object):
         self.revisions = revs
         self.summaries = summaries
 
-    def revision_sha1(self, idx):
-        return self.revisions[idx]
-
 
 class SelectCommitsDialog(QtGui.QDialog):
+
     def __init__(self, model,
                  parent=None, title=None, multiselect=True, syntax=True):
         QtGui.QDialog.__init__(self, parent)
@@ -61,42 +59,36 @@ class SelectCommitsDialog(QtGui.QDialog):
         self.close_button = QtGui.QPushButton(N_('Close'))
 
         # Make the list widget slighty larger
-        self.splitter = QtGui.QSplitter()
-        self.splitter.setOrientation(QtCore.Qt.Vertical)
-        self.splitter.setHandleWidth(defs.handle_width)
+        self.splitter = qtutils.splitter(Qt.Vertical,
+                                         self.commit_list, self.commit_text)
         self.splitter.setSizes([100, 150])
-        self.splitter.addWidget(self.commit_list)
-        self.splitter.addWidget(self.commit_text)
 
-        self.input_layout = QtGui.QHBoxLayout()
-        self.input_layout.setMargin(defs.no_margin)
-        self.input_layout.setSpacing(defs.spacing)
-        self.input_layout.addWidget(self.label)
-        self.input_layout.addWidget(self.revision)
-        self.input_layout.addWidget(self.select_button)
-        self.input_layout.addWidget(self.close_button)
+        self.input_layout = qtutils.hbox(defs.no_margin, defs.spacing,
+                                         self.label, self.revision,
+                                         self.select_button, self.close_button)
 
-        self.main_layout = QtGui.QVBoxLayout()
-        self.main_layout.setMargin(defs.margin)
-        self.main_layout.setSpacing(defs.margin)
-        self.main_layout.addWidget(self.splitter)
-        self.main_layout.addLayout(self.input_layout)
+        self.main_layout = qtutils.vbox(defs.margin, defs.margin,
+                                        self.splitter, self.input_layout)
         self.setLayout(self.main_layout)
 
         self.connect(self.commit_list,
                      SIGNAL('itemSelectionChanged()'), self.commit_sha1_selected)
 
+        self.connect(self.commit_list,
+                     SIGNAL('itemDoubleClicked(QListWidgetItem*)'),
+                     self.commit_sha1_double_clicked)
+
         qtutils.connect_button(self.select_button, self.accept)
         qtutils.connect_button(self.close_button, self.reject)
 
-        #self.setTabOrder(self.commit_list, self.commit_text)
-        #self.setTabOrder(self.commit_text, self.revision)
-        #self.setTabOrder(self.revision, self.select_button)
-        #self.setTabOrder(self.select_button, self.close_button)
-        #self.setTabOrder(self.close_button, self.commit_list)
-
         self.resize(700, 420)
 
+    def selected_commit(self):
+        return qtutils.selected_item(self.commit_list, self.model.revisions)
+
+    def selected_commits(self):
+        return qtutils.selected_items(self.commit_list, self.model.revisions)
+
     def select_commits(self):
         summaries = self.model.summaries
         if not summaries:
@@ -107,21 +99,23 @@ class SelectCommitsDialog(QtGui.QDialog):
         self.show()
         if self.exec_() != QtGui.QDialog.Accepted:
             return []
-        revs = self.model.revisions
-        return qtutils.selection_list(self.commit_list, revs)
+        return self.selected_commits()
 
     def commit_sha1_selected(self):
-        row, selected = qtutils.selected_row(self.commit_list)
+        sha1  = self.selected_commit()
+        selected = sha1 is not None
         self.select_button.setEnabled(selected)
         if not selected:
             self.commit_text.setText('')
             self.revision.setText('')
             return
-        # Get the sha1 and put it in the revision line
-        sha1 = self.model.revision_sha1(row)
         self.revision.setText(sha1)
         self.revision.selectAll()
-
         # Display the sha1's commit
         commit_diff = gitcmds.commit_diff(sha1)
         self.commit_text.setText(commit_diff)
+
+    def commit_sha1_double_clicked(self, item):
+        sha1 = self.selected_commit()
+        if sha1:
+            self.accept()
